<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 02/28/2009 *
 * ===============                              Last change: 02/28/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : config-functions.php                             *
 * -------------------------------------------------------------------- *
 * Short description : Many non-MySQL functions (also file access)      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Viele Nicht-MySQL-Funktionen (auch Dateizugriff) *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), '/inc') + 4) . '/security.php';
	require($INC);
}

// Init the config array
function initConfig () {
	// Init not if already found
	if ((isset($GLOBALS['config'])) && (count($GLOBALS['config']) >= 3)) {
		// Already initialized
		trigger_error(sprintf("[%s:%s] Configuration is already initialized.", __FUNCTION__, __LINE__));
	} // END - if

	// Set a minimum of configuration, required to by-pass some errors triggers in getConfig()
	$GLOBALS['config'] = array(
		'code_length'      => 0,
		'patch_level'      => 0,
		'last_update'      => time(),
		'DEFAULT_LANG'     => 'de',
		'DEBUG_MODE'       => 'N',
		'DEBUG_RESET'      => 'N',
		'DEBUG_MONTHLY'    => 'N',
		'DEBUG_WEEKLY'     => 'N',
		'DEBUG_REGEX'      => 'N',
		'ADMIN_REGISTERED' => 'N',
		'sql_count'        => 0,
		'num_templates'    => 0,
		'default_theme'    => 'default',
	);
}

// Getter for $GLOBALS['config'] entries
function getConfig ($configEntry) {
	// Default value
	$value = null;

	// Is the entry there?
	if (!isConfigEntrySet($configEntry)) {
		// Raise an error of missing entries
		trigger_error(sprintf("[%s:%s] Configuration entry <em>%s</em> is missing.", __FUNCTION__, __LINE__, $configEntry));
	} // END - if

	// Return it
	return $GLOBALS['config'][$configEntry];
}

// Setter for $GLOBALS['config'] entries
function setConfigEntry ($configEntry, $value) {
	// Secure the entry name
	if (function_exists('SQL_ESCAPE')) {
		// Use our secure function
		$configEntry = SQL_ESCAPE($configEntry);
	} else {
		// Use maybe insecure function
		$configEntry = smartAddSlashes($configEntry);
	}

	// And set it
	$GLOBALS['config'][$configEntry] = $value;
}

// Checks wether the given config entry is set
function isConfigEntrySet ($configEntry) {
	return (isset($GLOBALS['config'][$configEntry]));
}

// Merges $GLOBALS['config'] with data in given array
function mergeConfig ($newConfig) {
	$GLOBALS['config'] = merge_array(getConfigArray(), $newConfig);
}

// Increment or init with given value or 1 as default the given config entry
function incrementConfigEntry ($configEntry, $value=1) {
	// Increment it if set or init it with 1
	if (getConfig($configEntry) > 0) {
		$GLOBALS['config'][$configEntry] += $value;
	} else {
		$GLOBALS['config'][$configEntry] = $value;
	}
}

// Checks wether the configuration array is set so the config is loaded
function isConfigLoaded () {
	// Check all
	return ((isset($GLOBALS['config'])) && (is_array($GLOBALS['config'])) && (count($GLOBALS['config']) > 0));
}

// Load configuration and return it as an arry
function loadConfiguration ($no = '0') {
	// Check for cache extension, cache-array and if the requested configuration is in cache
	if ((isset($GLOBALS['cache_array'])) && (is_array($GLOBALS['cache_array'])) && (isset($GLOBALS['cache_array']['config'][$no])) && (is_array($GLOBALS['cache_array']['config'][$no]))) {
		// Load config from cache
		//* DEBUG: */ OUTPUT_HTML(gettype($GLOBALS['cache_array']['config'][$no])."<br />");
		foreach ($GLOBALS['cache_array']['config'][$no] as $key => $value) {
			setConfigEntry($key, $value);
		} // END - foreach

		// Count cache hits if exists
		if ((isConfigEntrySet('cache_hits')) && (EXT_IS_ACTIVE('cache'))) {
			incrementConfigEntry('cache_hits');
		} // END - if
	} elseif ((!EXT_IS_ACTIVE('cache')) || (!isset($GLOBALS['cache_array']['config'][$no]))) {
		// Load config from DB
		$result_config = SQL_QUERY_ESC("SELECT * FROM `{!_MYSQL_PREFIX!}_config` WHERE config=%d LIMIT 1",
			array(bigintval($no)), __FUNCTION__, __LINE__);

		// Is the config there?
		if (SQL_NUMROWS($result_config) == 1) {
			// Get config from database
			mergeConfig(SQL_FETCHARRAY($result_config));
		} // END - if

		// Free result
		SQL_FREERESULT($result_config);

		// Remember this config in the array
		$GLOBALS['cache_array']['config'][$no] = getConfigArray();
	}
}

// Getter for whole $GLOBALS['config'] array
function getConfigArray () {
	// Default is null
	$return = null;

	// Is the config set?
	if (isConfigLoaded()) {
		// Then use it
		$return = $GLOBALS['config'];
	} // END - if

	// Return result
	return $return;
}

// Updates an old inc/config.php to a inc/cache/config-local.php file
function updateOldConfigFile () {
	// Watch out for these lines and execute them as single command
	// @TODO Make this all better... :-/
	$watchLines = array(
		"define('SITE_KEY',"           => 'SITE_KEY',
		"define('DEFAULT_LANG',"       => 'DEFAULT_LANG',
		"define('warn_no_pass',"       => 'WARN_NO_PASS',
		"define('WRITE_FOOTER',"       => 'WRITE_FOOTER',
		"define('OUTPUT_MODE',"        => 'OUTPUT_MODE',
		"define('MAIN_TITLE',"         => 'MAIN_TITLE',
		"define('SLOGAN',"             => 'SLOGAN',
		"define('WEBMASTER',"          => 'WEBMASTER',
		"define('mxchange_installed'," => 'MXCHANGE_INSTALLED',
		"define('admin_registered',"   => 'ADMIN_REGISTERED',
		"define('_MYSQL_PREFIX',"      => '_MYSQL_PREFIX',
		"define('_TABLE_TYPE',"        => '_TABLE_TYPE',
		"define('_DB_TYPE',"           => '_DB_TYPE',
		"define('SMTP_HOSTNAME',"      => 'SMTP_HOSTNAME',
		"define('SMTP_USER'"           => 'SMTP_USER',
		"define('SMTP_PASSWORD',"      => 'SMTP_PASSWORD',
		"define('ENABLE_BACKLINK',"    => 'ENABLE_BACKLINK',
	);

	// Make these lower-case! (damn stupid code...)
	$lowerCase = array('WARN_NO_PASS', 'MXCHANGE_INSTALLED', 'ADMIN_REGISTERED');

	// These are still constants...
	// @TODO Rewrite these all to config entries, if somehow possible
	$constants = array('MAIN_TITLE', 'SLOGAN', 'WEBMASTER');

	// Special comments...
	$comments = array(
		'WARN_NO_PASS'       => 'NULLPASS-WARNING',
		'MXCHANGE_INSTALLED' => 'INSTALLED',
		'ADMIN_REGISTERED'   => 'ADMIN-SETUP',
		'_MYSQL_PREFIX'      => 'MYSQL-PREFIX',
		'_TABLE_TYPE'        => 'TABLE-TYPE',
		'_DB_TYPE'           => 'DATABASE-TYPE',
		'ENABLE_BACKLINK'    => 'BACKLINK',
		'host'               => 'MYSQL-HOST',
		'dbase'              => 'MYSQL-DBASE',
		'login'              => 'MYSQL-LOGIN',
		'password'           => 'MYSQL-PASSWORD'
	);

	// Copy template to new file destionation
	copyFileVerified(getConfig('PATH') . 'inc/config-local.php.dist', getConfig('PATH') . 'inc/cache/config-local.php', 0644);

	// First of all, load the old one!
	$oldConfig = explode("\n", readFromFile(getConfig('PATH') . 'inc/config.php'));

	// Now, analyze every entry
	$done = array();
	foreach ($oldConfig as $line) {
		// Check all watch lines
		foreach ($watchLines as $old => $new) {
			// Is the line found?
			if ((substr($line, 0, strlen($old)) == $old) && (!isset($done[$old]))) {
				// Entry found!
				//* DEBUG: */ OUTPUT_HTML(htmlentities($line) . " - FOUND!<br />");

				// Eval the line...
				eval($line);

				// Setting config entry is new default behaviour!
				$function = 'setConfigEntry';

				// Still some old constants left?
				if (in_array($new, $constants)) {
					// Then switch over...
					$function = 'define';
				} // END - if

				// Default comment
				$comment = str_replace('_', '-', $new);

				// Do we have a special comment?
				if (isset($comments[$new])) {
					// Then use it
					$comment = $comments[$new];
				} // END - if

				// Do we need to make $new lowercase?
				$oldNew = $new;
				if (in_array($new, $lowerCase)) {
					// Then do so... :)
					$new = strtolower($new);
				} // END - if

				/// ... and write it to the new config
				//* DEBUG: */ OUTPUT_HTML('function=' . $function . ',new=' . $new . ',comment=' . $comment . "<br />");
				changeDataInFile(getConfig('PATH') . 'inc/cache/config-local.php', $comment, $function . "('" . $oldNew . "', \"", "\");", constant($new), 0);
				//* DEBUG: */ OUTPUT_HTML("CHANGED!<br />");

				// Mark it as done
				$done[$old] = 1;
			} // END - if
		} // END - foreach
	} // END - foreach

	// By default the old array $MySQL was not found
	$found = false;

	// Analyze every entry again for the MySQL configuration
	foreach ($oldConfig as $line) {
		// Trim spaces
		$line = trim($line);

		// Is the $MySQL found?
		if (substr($line, 0, 6) == "\$MySQL") {
			// Okay found!
			$found = true;
		} elseif ($found === true) {
			// Now check this row
			if (substr($line, 0, 2) == ');') {
				// MySQL array is closed so stop looking for it
				break;
			} elseif (substr($line, 0, 2) == '//') {
				// Skip this line
				continue;
			}

			// Debug output only
			//* DEBUG: */ OUTPUT_HTML(htmlentities($line) . " - MySQL!<br />");

			// Split parts so we can check them and prepare them
			$parts = explode('=>', $line);
			$key = substr(trim($parts[0]), 1, -1); $value = substr(trim($parts[1]), 1, -2);

			// We can now save the right part in new config file
			changeDataInFile(getConfig('PATH') . 'inc/cache/config-local.php', $comments[$key], "	'".$key."'     => \"", "\",", $value, 0);
		}
	} // END - foreach

	// Finally remove old config file
	removeFile(getConfig('PATH') . 'inc/config.php');

	// Redirect to same URL to reload our new config
	redirectToUrl($_SERVER['REQUEST_URI']);
}

// Update config entries
function updateConfiguration ($entries, $values, $updateMode='') {
	// Do not update config in CSS mode
	if ((getOutputMode() == '1') || (getOutputMode() == -1)) {
		return;
	} // END - if

	// Do we have multiple entries?
	if (is_array($entries)) {
		// Walk through all
		$all = '';
		foreach ($entries as $idx => $entry) {
			// Update mode set?
			if (!empty($updateMode)) {
				// Update entry
				// @TODO Find a way for updating configuration here
				$all .= sprintf("`%s`=%s%s%s,", $entry, $entry, $updateMode, (float)$values[$idx]);
			} else {
				// Check if string or number
				if (($values[$idx] + 0) === $values[$idx]) {
					// Number detected
					$all .= sprintf("`%s`=%s,", $entry, (float)$values[$idx]);
				} elseif ($values[$idx] == 'UNIX_TIMESTAMP()') {
					// Function UNIX_TIMESTAMP() detected
					$all .= sprintf("`%s`=%s,", $entry, $values[$idx]);
				} else {
					// String detected
					$all .= sprintf("`%s`='%s',", $entry, SQL_ESCAPE($values[$idx]));
				}
			}

			// Set it in config as well
			setConfigEntry($entry, $values[$idx]);
		} // END - foreach

		// Remove last comma
		$entries = substr($all, 0, -1);
	} elseif (!empty($updateMode)) {
		// Update mode set
		// @TODO Find a way for updating configuration here
		$entries .= sprintf("=%s%s%s", $entries, $updateMode, (float)$values);
	} else {
		// Set it in config first
		setConfigEntry($entries, $values);

		// Regular entry to update
		$entries .= sprintf("='%s'", SQL_ESCAPE($values));
	}

	// Run database update
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "entries={$entries}");
	SQL_QUERY("UPDATE `{!_MYSQL_PREFIX!}_config` SET ".$entries." WHERE `config`=0 LIMIT 1", __FUNCTION__, __LINE__);

	// Get affected rows
	$affectedRows = SQL_AFFECTEDROWS();
	//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):entries={$entries},affectedRows={$affectedRows}<br />");

	// Rebuild cache
	rebuildCacheFiles('config', 'config');
}

// [EOF]
?>
