<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 03/25/2004 *
 * ===============                              Last change: 09/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : extensions.php                                   *
 * -------------------------------------------------------------------- *
 * Short description : Extension management                             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Erweiterungen-Management                         *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), '/inc') + 4) . '/security.php';
	require($INC);
}

// Load the extension and maybe found language and function files.
function LOAD_EXTENSION ($ext_name, $EXT_LOAD_MODE = '', $EXT_VER = '', $dry_run = false) {
	// Set current extension name
	EXT_SET_CURR_NAME($ext_name);

	// Init array
	INIT_INC_POOL();

	// Init EXT_UPDATE_DEPENDS
	EXT_INIT_UPDATE_DEPENDS();

	// Init current extension name list
	INIT_EXT_SQLS();

	// Is the extension already loaded?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "Loading extension {$ext_name}, mode={$EXT_LOAD_MODE}, ver={$EXT_VER}.");
	if ((isset($GLOBALS['ext_loaded']['ext'][$ext_name])) && (empty($EXT_LOAD_MODE))) {
		// Debug message
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Extension %s already loaded.", $ext_name));

		// Abort here
		return false;
	} // END - if

	// Construct include filename and FQFN for extension file
	$INC = sprintf("inc/extensions/ext-%s.php", $ext_name);
	$FQFN = getConfig('PATH') . $INC;

	// Is the extension file NOT there?
	if (!isIncludeReadable($INC)) {
		// Debug message
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Extension %s not found or not readable.", $ext_name));

		// Abort here
		return false;
	} // END - if

	// Load extension's own language file
	loadLanguageFile($ext_name);

	// Construct FQFN for functions file
	$funcsInclude = sprintf("inc/libs/%s_functions.php", $ext_name);

	// Is this include there?
	if ((isFileReadable($funcsInclude)) && (!isset($GLOBALS['ext_loaded']['funcs'][$ext_name]))) {
		// Then load it
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "--- Functions loaded.");
		$GLOBALS['ext_loaded']['funcs'][$ext_name] = true;
		loadIncludeOnce($funcsInclude);
	} elseif ((isDebugModeEnabled()) && ($ext_name != 'sql_patches') && (substr($ext_name, 0, 10) != 'admintheme')) {
		// No functions file is not so good...
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("NOTICE: Extension %s has no own functions file or we cannot read from it.",
			$ext_name
		));
	} // END - if

	// Extensions are not deprecated by default
	EXT_SET_DEPRECATED('N');

	// Extensions are not always active by default
	EXT_SET_ALWAYS_ACTIVE('N');

	// Extension update notes
	EXT_SET_UPDATE_NOTES('');

	// Include the extension file
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "Extension loaded.");
	require($FQFN);

	// Is this extension deprecated?
	if (EXT_GET_DEPRECATED() == 'Y') {
		// Deactivate the extension
		DEACTIVATE_EXTENSION($ext_name);

		// Abort here
		return false;
	} // END - if

	// Mark it as loaded in normal mode
	if (empty($EXT_LOAD_MODE)) {
		// Mark it now...
		$GLOBALS['ext_loaded']['ext'][$ext_name] = true;
	} // END - if

	// All fine!
	return true;
}

// Registeres an extension and possible update depencies
function REGISTER_EXTENSION ($ext_name, $task_id, $dry_run = false, $logout = true) {
	// Set current extension name
	EXT_SET_CURR_NAME($ext_name);

	//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME()." - ENTERED!<br />");
	// This shall never do a non-admin user or if the extension is active (already installed)
	if ((!IS_ADMIN()) || (EXT_IS_ACTIVE($ext_name))) {
		return false;
	} // END - if

	// When this extension is already in install/update phase, all is fine
	if (EXT_IS_REGISTER_RUNNING($ext_name)) {
		// Then abort here which is fine
		//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME()." - ALREADY!<br />");
		return true;
	} // END - if

	// This registration is running
	EXT_ADD_RUNNING_REGISTRATION($ext_name);

	// Init EXT_UPDATE_DEPENDS
	EXT_INIT_UPDATE_DEPENDS();

	// Is the task id zero? Then we need to auto-fix it here
	if ($task_id == 0) {
		// Try to find the task
		$task_id = DETERMINE_EXTENSION_TASK_ID(EXT_GET_CURR_NAME());

		// Still zero and not in dry-run?
		if (($task_id == 0) && (!$dry_run)) {
			// Then request a bug report
			debug_report_bug(sprintf("%s: task_id is still zero after DETERMINE_EXTENSION_TASK_ID(%s)",
			__FUNCTION__,
			EXT_GET_CURR_NAME()
			));
		} // END - if
	} // END - if

	// Init queries and notes
	INIT_EXT_SQLS();
	EXT_INIT_NOTES();

	// Init variables
	$ret = false;
	$test = false;
	INIT_INC_POOL();

	// By default we have no failures
	EXT_SET_REPORTS_FAILURE(false);

	// Does this extension exists?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME()."");
	if (LOAD_EXTENSION(EXT_GET_CURR_NAME(), 'register', '', $dry_run)) {
		// Set current extension name again
		EXT_SET_CURR_NAME($ext_name);

		// And run possible updates
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "".EXT_GET_CURR_NAME());
		$history = EXT_GET_VER_HISTORY();
		foreach ($history as $ver) {
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ext_name=".EXT_GET_CURR_NAME().", ext_ver={$ver}");
			// Load extension in update mode
			LOAD_EXTENSION(EXT_GET_CURR_NAME(), 'update', $ver, $dry_run);

			// Add update notes to our output
			ADD_EXTENSION_NOTES($ver);
		} // END - foreach

		// Does this extension depends on an outstanding update of another update?
		for ($dmy = EXT_GET_UPDATE_ITERATOR(); EXT_GET_UPDATE_ITERATOR() < EXT_COUNT_UPDATE_DEPENDS();) {
			// Get next update
			$ext_update = EXT_GET_ITERATOR_UPDATE_DEPENDS();

			// Increment here to avoid endless loop
			EXT_INCREMENT_UPDATE_INTERATOR();

			// Check for required file
			if (LOAD_EXTENSION($ext_update, 'register', '', $dry_run)) {
				// Set current extension name again
				EXT_SET_CURR_NAME($ext_name);

				// If versions mismatch update extension first
				$ext_ver = GET_EXT_VERSION($ext_update);

				// Extension version set? If empty the extension is not registered
				if (empty($ext_ver)) {
					// Extension not registered so far so first load task's ID...
					$task = DETERMINE_EXTENSION_TASK_ID($ext_update);

					// Entry found?
					if ($task > 0) {
						// Try to register the extension
						//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME().":ext_update=".$ext_update.",taskId=".$task."<br />");
						$test = REGISTER_EXTENSION($ext_update, $task, $dry_run, false);

						// Reset extension name
						EXT_SET_CURR_NAME($ext_name);
						//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME().':'); var_dump($test);
					} // END - if
				} elseif ($ext_ver != EXT_GET_VERSION()) {
					// Ok, update this extension now
					EXTENSION_UPDATE($ext_update, $ext_ver, $dry_run);

					// All okay!
					$test = true;
				} else {
					// Nothing to register / update before...
					$test = true;
				}
			} else {
				// Required file for update does not exists!
				$test = true;
				// But this is fine for the first time...
			}

			// Restore the current extension name
			EXT_SET_CURR_NAME($ext_name);
		} // END - for

		// Is there no update?
		if (EXT_COUNT_UPDATE_DEPENDS(EXT_GET_CURR_NAME()) == 0) {
			// Then test is passed!
			$test = true;
		} // END - if

		// Switch back to register mode
		$EXT_LOAD_MODE = 'register';

		// Remains true if extension registration reports no failures
		//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME().':'); var_dump($test);
		$test = (($test === true) && (EXT_GET_REPORTS_FAILURE() === false));
		//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME().':'); var_dump($test);

		// Does everthing before wents ok?
		if ($test === true) {
			// "Dry-run-mode" activated?
			if ((!$dry_run) && (!EXT_IS_ON_REMOVAL_LIST())) {
				//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ext_name=".EXT_GET_CURR_NAME());
				// Init SQLs and transfer ext->generic
				INIT_SQLS();
				SET_SQLS(GET_EXT_SQLS());

				// Run installation pre-installation filters
				runFilterChain('pre_extension_installed', array('dry_run' => $dry_run));

				// Register extension
				//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:insert=".EXT_GET_CURR_NAME().'/'.EXT_GET_VERSION()." - INSERT!<br />");
				SQL_QUERY_ESC("INSERT INTO `{!_MYSQL_PREFIX!}_extensions` (ext_name, ext_active, ext_version) VALUES ('%s','%s','%s')",
					array(EXT_GET_CURR_NAME(), EXT_GET_ALWAYS_ACTIVE(), EXT_GET_VERSION()), __FUNCTION__, __LINE__);

				// Remove cache file(s) if extension is active
				runFilterChain('post_extension_installed', array('ext_name' => EXT_GET_CURR_NAME(), 'task_id' => $task_id));

				// Remove all SQL commands
				UNSET_SQLS();

				// In normal mode return a true on success
				$ret = true;
			} elseif ($dry_run) {
				// Init SQLs and transfer ext->generic
				INIT_SQLS();
				SET_SQLS(GET_EXT_SQLS());

				// Rewrite SQL command to keep { and } inside for dry-run
				foreach (GET_SQLS() as $key => $sql) {
					$sql = str_replace('{', "&#123;", str_replace('}', "&#125;", $sql));
					SET_SQL_KEY($key, $sql);
				} // END - foreach

				// In  "dry-run" mode return array with all SQL commands
				$ret = GET_SQLS();

				// Remove all SQL commands
				UNSET_SQLS();
			} else {
				// Extension has been removed for updates, so all is fine!
				$ret = true;
			}
		} else {
			// No, an error occurs while registering extension :-(
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "".EXT_GET_CURR_NAME());
			$ret = false;
		}
	} elseif (($task_id > 0) && (EXT_GET_CURR_NAME() != '')) {
		//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME()."<br />");
		// Remove task from system when id and extension's name is valid
		SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{!_MYSQL_PREFIX!}_task_system` WHERE `id`=%s AND `status`='NEW' LIMIT 1",
			array(bigintval($task_id)), __FUNCTION__, __LINE__);
	}

	// Is this the sql_patches?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ':'.EXT_GET_CURR_NAME()."/{$EXT_LOAD_MODE}");
	if ((EXT_GET_CURR_NAME() == 'sql_patches') && (($EXT_LOAD_MODE == 'register') || ($EXT_LOAD_MODE == 'remove')) && (!$dry_run) && ($test)) {
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": LOAD!");
		if ($logout === true) {
			// Then redirect to logout
			redirectToUrl('modules.php?module=admin&amp;logout=1&amp;' . $EXT_LOAD_MODE . '=sql_patches');
		} else {
			// Add temporary filter
			registerFilter('shutdown', 'REDIRECT_TO_LOGOUT_SQL_PATCHES', true, true);
			$GLOBALS['ext_load_mode'] = $EXT_LOAD_MODE;
		}
	} // END - if

	// Return status code
	//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME()." - LEFT!<br />");
	//* DEBUG: */ var_dump($ret);
	return $ret;
}

// Run SQL queries for given extension id
// @TODO Change from ext_id to ext_name (not just even the variable! ;-) )
function EXTENSION_RUN_SQLS ($ext_id, $load_mode) {
	// This shall never do a non-admin user!
	if (!IS_ADMIN()) return false;

	// Get extension's name
	$ext_name = GET_EXT_NAME($ext_id);

	// If it is not set then maybe there is no extension for that ID number
	if (empty($ext_name)) {
		// We should fix these all!
		debug_report_bug(__FUNCTION__ . ': ext_name is empty. ext_id=' . $ext_id);
	} // END - if

	// Set current SQL name
	EXT_SET_CURR_NAME($ext_name);

	// Init EXT_UPDATE_DEPENDS
	EXT_INIT_UPDATE_DEPENDS();

	// Init array
	INIT_EXT_SQLS();

	// By default no SQL has been executed
	$sqlRan = false;

	// Load extension in detected mode
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ":ext_name[{$ext_id}]=".EXT_GET_CURR_NAME()."");
	LOAD_EXTENSION(EXT_GET_CURR_NAME(), $load_mode, '', false);

	// Init these SQLs
	INIT_SQLS();
	SET_SQLS(GET_EXT_SQLS());

	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ":SQLs::count=".COUNT_SQLS()."");
	if ((IS_SQLS_VALID() && (COUNT_SQLS() > 0))) {
		// Run any filters depending on the action here
		runFilterChain('extension_' . $load_mode, $ext_name);

		// Run SQL commands...
		runFilterChain('run_sqls');

		// Removal mode?
		if ($load_mode == 'remove') {
			// Delete this extension (remember to remove it from your server *before* you click on welcome!
			SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{!_MYSQL_PREFIX!}_extensions` WHERE `ext_name`='%s' LIMIT 1",
				array(EXT_GET_CURR_NAME()), __FUNCTION__, __LINE__);
		} // END - if
	} // END - if

	// Remove cache file(s) if extension is active
	if (((EXT_IS_ACTIVE('cache')) || (GET_EXT_VERSION('cache') != '')) && (((SQL_AFFECTEDROWS() == 1)) || ($sqlRan === true) || ($load_mode == 'activate') || ($load_mode == 'deactivate'))) {
		// Run filters
		runFilterChain('post_extension_run_sql', EXT_GET_CURR_NAME());
	} // END - if

	// Is this the sql_patches?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": id=".$ext_id.",currName=".EXT_GET_CURR_NAME().",loadMode=".$load_mode);
	if ((EXT_GET_CURR_NAME() == 'sql_patches') && (($load_mode == 'register') || ($load_mode == 'remove'))) {
		// Then redirect to logout
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": LOAD!");
		redirectToUrl('modules.php?module=admin&amp;logout=1&amp;' . $load_mode . '=sql_patches');
	} // END - if
}

// Check if given extension is active
function EXT_IS_ACTIVE ($ext_name) {
	// Extensions are all inactive during installation
	if ((!isInstalled()) || (isInstalling()) || (empty($ext_name))) return false;

	// Not active is the default
	$active = 'N';

	// Check cache
	if (isset($GLOBALS['cache_array']['extensions']['ext_active'][$ext_name])) {
		// Load from cache
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "CACHE! ext_name={$ext_name}");
		$active = $GLOBALS['cache_array']['extensions']['ext_active'][$ext_name];

		// Count cache hits
		incrementConfigEntry('cache_hits');
	} elseif (isset($GLOBALS['ext_loaded'][$ext_name])) {
		// @TODO Extension is loaded, what next?
		app_die(__FUNCTION__, __LINE__, "LOADED:$ext_name");
	} elseif (($ext_name == 'cache') || (GET_EXT_VERSION('cache') == '')) {
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "DB! ext_name={$ext_name}");
		// Load from database
		$result = SQL_QUERY_ESC("SELECT `ext_active` FROM `{!_MYSQL_PREFIX!}_extensions` WHERE `ext_name`='%s' LIMIT 1",
			array($ext_name), __FUNCTION__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result) == 1) {
			// Load entry
			list($active) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);

		// Write cache array
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ext_name=".$ext_name."[DB]: {$active}");
		$GLOBALS['cache_array']['extensions']['ext_active'][$ext_name] = $active;
	} else {
		// Extension not active!
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ext_name=".$ext_name.": Not active!");
		$GLOBALS['cache_array']['extensions']['ext_active'][$ext_name] = 'N';
	}

	// Debug message
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ext_name={$ext_name},active={$active}");

	// Is this extension activated? (For admins we always have active extensions...)
	return ($active == 'Y');
}
// Get version from extensions
function GET_EXT_VERSION ($ext_name) {
	// By default no extension is found
	$ext_ver = false;

	// Empty extension name should be fixed!
	if (empty($ext_name)) {
		// Please report this bug!
		debug_report_bug(__FUNCTION__ . ': ext_name is empty which is not allowed here.');
	} // END - if

	// Extensions are all inactive during installation
	if ((!isInstalled()) || (isInstalling())) return "";
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": ext_name={$ext_name}");

	// Is the cache written?
	if (isset($GLOBALS['cache_array']['extensions']['ext_version'][$ext_name])) {
		// Load data from cache
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": CACHE!");
		$ext_ver = $GLOBALS['cache_array']['extensions']['ext_version'][$ext_name];

		// Count cache hits
		incrementConfigEntry('cache_hits');
	} elseif (!isCacheInstanceValid()) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT `ext_version` FROM `{!_MYSQL_PREFIX!}_extensions` WHERE `ext_name`='%s' LIMIT 1",
			array($ext_name), __FUNCTION__, __LINE__);
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": DB - ".SQL_NUMROWS($result)."");

		// Is the extension there?
		if (SQL_NUMROWS($result) == 1) {
			// Load entry
			list($ext_ver) = SQL_FETCHROW($result);
		} elseif (isDebugModeEnabled()) {
			// Not found!
			DEBUG_LOG(__FUNCTION__, __LINE__, sprintf(": Cannot find extension %s in database!", $ext_name));
		}

		// Free result
		SQL_FREERESULT($result);

		// Set cache
		$GLOBALS['cache_array']['extensions']['ext_version'][$ext_name] = $ext_ver;
	}

	// Return result
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, ": ret={$ext_ver}");
	return $ext_ver;
}

// Updates a given extension with current extension version to latest version
function EXTENSION_UPDATE ($ext_name, $ext_ver, $dry_run = false) {
	// Only admins are allowed to update extensions
	if ((!IS_ADMIN()) || (empty($ext_name))) return false;

	// Set current SQL name
	EXT_SET_CURR_NAME($ext_name);

	// Init arrays
	INIT_EXT_SQLS();
	EXT_INIT_NOTES();
	INIT_INC_POOL();

	// Load extension in test mode
	LOAD_EXTENSION($ext_name, 'test', $ext_ver, $dry_run);

	// Save version history
	$history = EXT_GET_VER_HISTORY();

	// Remove old SQLs array to prevent possible bugs
	INIT_EXT_SQLS();

	// Check if version is updated
	if (((EXT_GET_VERSION() != $ext_ver) || ($dry_run)) && (is_array($history))) {
		// Search for starting point
		$start = array_search($ext_ver, $history);

		// And load SQL queries in order of version history
		for ($idx = ($start + 1); $idx < count($history); $idx++) {
			// Set current extension name
			//* DEBUG: */ OUTPUT_HTML(__FUNCTION__.'['.__LINE__.':] ext_name='.$ext_name."<br />");
			EXT_SET_CURR_NAME($ext_name);

			// Set extension version
			$GLOBALS['cache_array']['update_ver'][EXT_GET_CURR_NAME()] = $history[$idx];

			// Load again...
			LOAD_EXTENSION(EXT_GET_CURR_NAME(), 'update', $GLOBALS['cache_array']['update_ver'][EXT_GET_CURR_NAME()], $dry_run);

			// Get all depencies
			$depencies = EXT_GET_UPDATE_DEPENDS();

			// Nothing to apply?
			if (count($depencies) > 0) {
				// Apply all extension depencies
				foreach ($depencies as $ext_depend) {
					// Set it as current
					EXT_SET_CURR_NAME($ext_depend);

					// Is the extension there?
					if (GET_EXT_VERSION($ext_depend) != '') {
						// Update another extension first!
						$test = EXTENSION_UPDATE($ext_depend, GET_EXT_VERSION($ext_depend), $dry_run);
					} else {
						// Register new extension
						$test = REGISTER_EXTENSION($ext_depend, 0, $dry_run, false);
					}
				} // END - foreach

				// Set name back
				EXT_SET_CURR_NAME($ext_name);
			} // END - if

			// Add notes
			ADD_EXTENSION_NOTES($GLOBALS['cache_array']['update_ver'][EXT_GET_CURR_NAME()]);
		} // END - for

		// In real-mode execute any existing includes
		if (!$dry_run) {
			$GLOBALS['cache_array']['inc_pool'][EXT_GET_CURR_NAME()] = GET_INC_POOL();
			runFilterChain('load_includes');
			SET_INC_POOL($GLOBALS['cache_array']['inc_pool'][EXT_GET_CURR_NAME()]);
			unset($GLOBALS['cache_array']['inc_pool'][EXT_GET_CURR_NAME()]);
		} // END - if

		// Init these SQLs
		INIT_SQLS();
		SET_SQLS(GET_EXT_SQLS());

		// Run SQLs
		runFilterChain('run_sqls', array('dry_run' => $dry_run));

		if (!$dry_run) {
			// Create task
			CREATE_EXTENSION_UPDATE_TASK(getCurrentAdminId(), EXT_GET_CURR_NAME(), $GLOBALS['cache_array']['update_ver'][EXT_GET_CURR_NAME()], SQL_ESCAPE(EXT_GET_NOTES(EXT_GET_NOTES())));

			// Update extension's version
			SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_extensions` SET `ext_version`='%s' WHERE `ext_name`='%s' LIMIT 1",
				array($GLOBALS['cache_array']['update_ver'][EXT_GET_CURR_NAME()], EXT_GET_CURR_NAME()), __FUNCTION__, __LINE__);

			// Remove arrays
			UNSET_SQLS();
			unset($GLOBALS['cache_array']['update_ver'][EXT_GET_CURR_NAME()]);

			// Run filters on success extension update
			runFilterChain('extension_update', EXT_GET_CURR_NAME());
		} // END - if
	} // END - if
}

// Output verbose SQL table for extension
function EXTENSION_VERBOSE_TABLE ($queries = array(), $title = '', $dashed = '', $switch = false, $width = '100%') {
	// Empty title?
	if (empty($title)) {
		// Then fix it to default
		$title = getMessage('ADMIN_SQLS_EXECUTED_ON_REMOVAL');
	} // END - if

	// Are there some queries in $queries?
	if (count($queries) > 0) {
		// Then use them instead!
		SET_SQLS($queries);
	} // END - if

	// Init variables
	$SW = 2; $i = 1;
	$OUT = '';

	// Do we have queries?
	if ((IS_SQLS_VALID()) && (GET_EXT_VERSION('sql_patches') >= '0.0.7') && (getConfig('verbose_sql') == 'Y')) {
		foreach (GET_SQLS() as $idx => $sql) {
			// Trim out spaces
			$sql = trim($sql);

			// Output command if set
			if (!empty($sql)) {
				// Prepare output for template
				$content = array(
					'sw'  => $SW,
					'i'   => $i,
					'sql' => $sql
				);

				// Load row template
				$OUT .= LOAD_TEMPLATE("admin_ext_sql_row", true, $content);

				// Switch color and count up
				$SW = 3 - $SW;
				$i++;
			} // END - if
		} // END - foreach

		// Prepare content for template
		$content = array(
			'width'  => $width,
			'dashed' => $dashed,
			'title'  => $title,
			'out'    => $OUT
		);

		// Load main template
		$OUT = LOAD_TEMPLATE("admin_ext_sql_table", true, $content);
	} elseif ((GET_EXT_VERSION('sql_patches') >= '0.0.7') && (getConfig('verbose_sql') == 'Y')) {
		// No addional SQL commands to run
		$OUT = LOAD_TEMPLATE('admin_settings_saved', true, getMessage('ADMIN_NO_ADDITIONAL_SQLS'));
	} // END - if

	// Return output
	return $OUT;
}

// Get extension name from id
function GET_EXT_NAME ($ext_id) {
	// Init extension name
	$ret = '';

	// Is cache there?
	if (isset($GLOBALS['cache_array']['extensions']['ext_name'][$ext_id])) {
		// Load from cache
		$ret = $GLOBALS['cache_array']['extensions']['ext_name'][$ext_id];

		// Count cache hits
		incrementConfigEntry('cache_hits');
	} elseif (!EXT_IS_ACTIVE('cache')) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT ext_name FROM `{!_MYSQL_PREFIX!}_extensions` WHERE `id`=%s LIMIT 1",
			array(bigintval($ext_id)), __FUNCTION__, __LINE__);
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}

	// Return the extension name
	return $ret;
}

// Get extension id from name
function GET_EXT_ID ($ext_name) {
	// Init ID number
	$ret = 0;
	if (isset($GLOBALS['cache_array']['extensions']['ext_id'][$ext_name])) {
		// Load from cache
		$ret = $GLOBALS['cache_array']['extensions']['ext_id'][$ext_name];

		// Count cache hits
		incrementConfigEntry('cache_hits');
	} elseif (!EXT_IS_ACTIVE('cache')) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT `id` FROM `{!_MYSQL_PREFIX!}_extensions` WHERE `ext_name`='%s' LIMIT 1",
			array($ext_name), __FUNCTION__, __LINE__);
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}

	// Return value
	return $ret;
}

// Activate given extension
function ACTIVATE_EXTENSION ($ext_name) {
	// Activate the extension
	SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_extensions` SET `ext_active`='Y' WHERE `ext_name`='%s' LIMIT 1",
	array($ext_name), __FUNCTION__, __LINE__);

	// Extension has been activated?
	if (SQL_AFFECTEDROWS() == 1) {
		// Then run all queries
		EXTENSION_RUN_SQLS(GET_EXT_ID($ext_name), 'activate');
	} // END - if
}

// Deactivate given extension
function DEACTIVATE_EXTENSION($ext_name) {
	// Activate the extension
	SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_extensions` SET `ext_active`='N' WHERE `ext_name`='%s' LIMIT 1",
	array($ext_name), __FUNCTION__, __LINE__);

	// Extension has been activated?
	if (SQL_AFFECTEDROWS() == 1) {
		// Then run all queries
		EXTENSION_RUN_SQLS(GET_EXT_ID($ext_name), 'deactivate');

		// Create new task
		CREATE_EXTENSION_DEACTIVATION_TASK($ext_name);

		// Notify the admin
		sendAdminNotification(
		getMessage('ADMIN_SUBJECT_EXTENSION_DEACTIVATED'),
			'admin_ext_deactivated',
		array('ext_name' => $ext_name)
		);
	} // END - if
}

// Checks wether the extension is older than given
function EXT_VERSION_IS_OLDER ($ext_name, $ext_ver) {
	// Get current extension version
	$currVersion = GET_EXT_VERSION($ext_name);

	// Remove all dots from both versions
	$currVersion = str_replace('.', '', $currVersion);
	$ext_ver = str_replace('.', '', $ext_ver);

	// Now compare both and return the result
	return ($currVersion < $ext_ver);
}

// Creates a new task for updated extension
function CREATE_EXTENSION_UPDATE_TASK ($admin_id, $ext_name, $ext_ver, $notes) {
	// Create subject line
	$subject = '[UPDATE-'.$ext_name.'-'.$ext_ver.':] {--ADMIN_UPDATE_EXT_SUBJ--}';

	// Is the extension there?
	if (GET_EXT_VERSION($ext_name) != '') {
		// Check if task is not there
		if (DETERMINE_TASK_ID_BY_SUBJECT($subject) == 0) {
			// Task not created so it's a brand-new extension which we need to register and create a task for!
			createNewTask($subject, $notes, 'EXTENSION_UPDATE', 0, $admin_id);
		} // END - if
	} // END - if
}

// Creates a new task for newly installed extension
function CREATE_NEW_EXTENSION_TASK ($admin_id, $subject, $ext) {
	// Not installed and do we have created a task for the admin?
	if ((DETERMINE_TASK_ID_BY_SUBJECT($subject) == 0) && (GET_EXT_VERSION($ext) == '')) {
		// Set default message if ext-foo is missing
		$message = sprintf(getMessage('ADMIN_EXT_TEXT_FILE_MISSING'), $ext);

		// Template file
		$tpl = sprintf("%stemplates/%s/html/ext/ext_%s.tpl",
			getConfig('PATH'),
			getLanguage(),
			$ext
		);

		// Load text for task if found
		if (isFileReadable($tpl)) {
			// Load extension's own text template (HTML!)
			$message = LOAD_TEMPLATE('ext_' . $ext, true);
		} else {
			// Write this in debug.log as well
			DEBUG_LOG(__FUNCTION__, __LINE__, $message);
		}

		// Task not created so it's a brand-new extension which we need to register and create a task for!
		createNewTask($subject, $message, 'EXTENSION_UPDATE', 0, $admin_id, false);
	} // END - if
}

// Creates a task for automatically deactivated (deprecated) extension
function CREATE_EXTENSION_DEACTIVATION_TASK ($ext) {
	// Create subject line
	$subject = sprintf("[%s:] %s", $ext, getMessage('TASK_SUBJ_EXTENSION_DEACTIVATED'));

	// Not installed and do we have created a task for the admin?
	if ((DETERMINE_TASK_ID_BY_SUBJECT($subject) == 0) && (GET_EXT_VERSION($ext) != '')) {
		// Task not created so add it
		createNewTask($subject, SQL_ESCAPE(LOAD_TEMPLATE('task_ext_deactivated', true, $ext)), 'EXTENSION_DEACTIVATION');
	} // END - if
}

// Checks if the module has a menu
function MODULE_HAS_MENU ($mod, $forceDb = false) {
	// All is false by default
	$ret = false;

	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "mod={$mod},cache=".GET_EXT_VERSION('cache'));
	if (GET_EXT_VERSION('cache') >= '0.1.2') {
		// Cache version is okay, so let's check the cache!
		if (isset($GLOBALS['cache_array']['modules']['has_menu'][$mod])) {
			// Check module cache and count hit
			$ret = ($GLOBALS['cache_array']['modules']['has_menu'][$mod] == 'Y');
			incrementConfigEntry('cache_hits');
		} elseif (isset($GLOBALS['cache_array']['extensions']['ext_menu'][$mod])) {
			// Check cache and count hit
			$ret = ($GLOBALS['cache_array']['extensions']['ext_menu'][$mod] == 'Y');
			incrementConfigEntry('cache_hits');
		} elseif ((IS_ADMIN()) && ($mod == 'admin')) {
			// Admin module has always a menu!
			$ret = true;
		}
	} elseif ((GET_EXT_VERSION('sql_patches') >= '0.3.6') && ((!EXT_IS_ACTIVE('cache')) || ($forceDb === true))) {
		// Check database for entry
		$result = SQL_QUERY_ESC("SELECT `has_menu` FROM `{!_MYSQL_PREFIX!}_mod_reg` WHERE `module`='%s' LIMIT 1",
			array($mod), __FUNCTION__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result) == 1) {
			// Load "has_menu" column
			list($has_menu) = SQL_FETCHROW($result);

			// Fake cache... ;-)
			$GLOBALS['cache_array']['extensions']['ext_menu'][$mod] = $has_menu;

			// Does it have a menu?
			$ret = ($has_menu == 'Y');
		} // END  - if

		// Free memory
		SQL_FREERESULT($result);
	} elseif (GET_EXT_VERSION('sql_patches') == '') {
		// No sql_patches installed, so maybe in admin area or no admin registered?
		$ret = (((IS_ADMIN()) || (!isAdminRegistered())) && ($mod == 'admin')); // Then there is a menu!
	}

	// Return status
	//* DEBUG: */ OUTPUT_HTML(__FUNCTION__."[".__LINE__."]:currName=".EXT_GET_CURR_NAME().':'; var_dump($ret));
	return $ret;
}

// Determines the task id for given extension
function DETERMINE_EXTENSION_TASK_ID ($ext_name) {
	// Default is not found
	$task_id = 0;

	// Search for extension task's id
	$result = SQL_QUERY_ESC("SELECT `id` FROM `{!_MYSQL_PREFIX!}_task_system` WHERE `task_type`='EXTENSION' AND `subject`='[%s:]' LIMIT 1",
		array($ext_name), __FUNCTION__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Task found so load task's ID and register extension...
		list($task_id) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return it
	return $task_id;
}

// Determines the task id for given subject
function DETERMINE_TASK_ID_BY_SUBJECT ($subject) {
	// Default is not found
	$task_id = 0;

	// Search for task id
	$result = SQL_QUERY_ESC("SELECT `id` FROM `{!_MYSQL_PREFIX!}_task_system` WHERE `subject` LIKE '%s%%' LIMIT 1",
		array($subject), __FUNCTION__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Task found so load task's ID and register extension...
		list($task_id) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return it
	return $task_id;
}

// Add updates notes for given version
function ADD_EXTENSION_NOTES ($ver) {
	// Init notes/content
	$out = ''; $content = array();

	// Is do we have verbose output enabled?
	if ((getConfig('verbose_sql') == 'Y') || (!EXT_IS_ACTIVE('sql_patches'))) {
		// Update notes found?
		if (EXT_GET_UPDATE_NOTES() != '') {
			// Update notes found
			$content = array(
				'ver'   => $ver,
				'notes' => EXT_GET_UPDATE_NOTES()
			);

			// Reset them
			EXT_SET_UPDATE_NOTES('');
		} elseif (($ver == '0.0') || ($ver == '0.0.0')) {
			// Initial release
			$content = array(
				'ver'   => $ver,
				'notes' => getMessage('INITIAL_RELEASE')
			);
		} else {
			// No update notes found!
			$content = array(
				'ver'   => $ver,
				'notes' => getMessage('NO_UPDATE_NOTES')
			);
		}

		// Load template
		$out = LOAD_TEMPLATE('admin_ext_notes', true, $content);
	} // END - if

	// Add the notes
	EXT_APPEND_NOTES($out);
}

// Getter for CSS files array
function EXT_GET_CSS_FILES () {
	// By default no additional CSS files are found
	$cssFiles = array();

	// Is the array there?
	if (isset($GLOBALS['css_files'])) {
		// Then use it
		$cssFiles = $GLOBALS['css_files'];
	} // END - if

	// Return array
	return $cssFiles;
}

// Init CSS files array
function EXT_INIT_CSS_FILES () {
	// Simply init it
	$GLOBALS['css_files'] = array();
}

// Add new entry
function EXT_ADD_CSS_FILE ($file) {
	// Is the array there?
	if (!isset($GLOBALS['css_files'])) {
		// Then auto-init them
		EXT_INIT_CSS_FILES();
	} // END - if

	// Add the entry
	$GLOBALS['css_files'][] = $file;
}

// Setter for EXT_ALWAYS_ACTIVE flag
function EXT_SET_ALWAYS_ACTIVE ($active) {
	$GLOBALS['ext_always_active'][EXT_GET_CURR_NAME()] = (string) $active;
}

// Getter for EXT_ALWAYS_ACTIVE flag
function EXT_GET_ALWAYS_ACTIVE () {
	return $GLOBALS['ext_always_active'][EXT_GET_CURR_NAME()];
}

// Setter for EXT_VERSION flag
function EXT_SET_VERSION ($version) {
	$GLOBALS['ext_version'][EXT_GET_CURR_NAME()] = (string) $version;
}

// Getter for EXT_VERSION flag
function EXT_GET_VERSION () {
	return $GLOBALS['ext_version'][EXT_GET_CURR_NAME()];
}

// Setter for EXT_DEPRECATED flag
function EXT_SET_DEPRECATED ($deprecated) {
	$GLOBALS['ext_deprecated'][EXT_GET_CURR_NAME()] = (string) $deprecated;
}

// Getter for EXT_DEPRECATED flag
function EXT_GET_DEPRECATED () {
	return $GLOBALS['ext_deprecated'][EXT_GET_CURR_NAME()];
}

// Setter for EXT_UPDATE_DEPENDS flag
function EXT_ADD_UPDATE_DEPENDS ($updateDepends) {
	// Is the update depency empty? (NEED TO BE FIXED!)
	if (empty($updateDepends)) {
		// Please report this bug!
		debug_report_bug("updateDepends is left empty!");
	} // END - if

	// Is it not yet added?
	if (!in_array($updateDepends, $GLOBALS['ext_running_updates'])) {
		//* DEBUG */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME().'/'.$updateDepends);
		// Add it to the list of extension update depencies map
		$GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()][] = (string) $updateDepends;

		// Remember it in the list of running updates
		$GLOBALS['ext_running_updates'][] = $updateDepends;
	} // END - if
}

// Checks wether the given extension registration is in progress
function EXT_IS_REGISTER_RUNNING ($ext_name) {
	return ((isset($GLOBALS['ext_register_running'])) && (in_array($ext_name, $GLOBALS['ext_register_running'])));
}

// Init EXT_UPDATE_DEPENDS flag
function EXT_INIT_UPDATE_DEPENDS () {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME());

	// Init update depency map automatically if not found
	if (!EXT_IS_UPDATE_DEPENDS_INIT()) {
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME()." - INIT!");
		$GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()] = array();
	} // END - if

	// Init running updates array
	EXT_INIT_RUNNING_UPDATES();
}

// Adds an extension as "registration in progress"
function EXT_ADD_RUNNING_REGISTRATION ($ext_name) {
	// Is it running?
	if (!EXT_IS_REGISTER_RUNNING($ext_name)) {
		// Then add it!
		$GLOBALS['ext_register_running'][] = $ext_name;
	} // END - if
}

// Checks wether EXT_UPDATE_DEPENDS is initialized
function EXT_IS_UPDATE_DEPENDS_INIT () {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME());
	return (isset($GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()]));
}

// Initializes the list of running updates
function EXT_INIT_RUNNING_UPDATES () {
	// Auto-init ext_running_updates
	if (!isset($GLOBALS['ext_running_updates'])) {
		$GLOBALS['ext_running_updates'] = array();
		$GLOBALS['ext_register_running'] = array();
	} // END - if
}

// Getter for EXT_UPDATE_DEPENDS flag
function EXT_GET_UPDATE_DEPENDS () {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME());
	return $GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()];
}

// Getter for next iterator depency
function EXT_GET_ITERATOR_UPDATE_DEPENDS () {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME());
	return ($GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()][EXT_GET_UPDATE_ITERATOR()]);
}

// Counter for extension update depencies
function EXT_COUNT_UPDATE_DEPENDS () {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME());
	return count($GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()]);
}

// Removes given extension from update denpency list
function EXT_REMOVE_UPDATE_DEPENDS ($ext_name) {
	// Look it up
	$key = array_search($ext_name, EXT_GET_UPDATE_DEPENDS());

	// Is it valid?
	if ($key !== false) {
		// Then remove it
		unset($GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()][$key]);

		// And sort the array
		ksort($GLOBALS['ext_update_depends'][EXT_GET_CURR_NAME()]);
	} // END - if
}

// Init iterator for update depencies
function EXT_INIT_UPDATE_ITERATOR () {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME());
	$GLOBALS['ext_depend_iterator'][EXT_GET_CURR_NAME()] = 0;
}

// Getter for depency iterator
function EXT_GET_UPDATE_ITERATOR () {
	// Auto-init iterator
	if (!isset($GLOBALS['ext_depend_iterator'][EXT_GET_CURR_NAME()])) EXT_INIT_UPDATE_ITERATOR();

	// Return it
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME().'/'.$GLOBALS['ext_depend_iterator'][EXT_GET_CURR_NAME()]);
	return $GLOBALS['ext_depend_iterator'][EXT_GET_CURR_NAME()];
}

// Increments the update iterator
function EXT_INCREMENT_UPDATE_INTERATOR () {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "currName=".EXT_GET_CURR_NAME());
	$GLOBALS['ext_depend_iterator'][EXT_GET_CURR_NAME()]++;
}

// Setter for EXT_REPORTS_FAILURE flag
function EXT_SET_REPORTS_FAILURE ($reportsFailure) {
	$GLOBALS['ext_reports_failure'] = (bool) $reportsFailure;
}

// Getter for EXT_REPORTS_FAILURE flag
function EXT_GET_REPORTS_FAILURE () {
	return $GLOBALS['ext_reports_failure'];
}

// Setter for EXT_VER_HISTORY flag
function EXT_SET_VER_HISTORY ($verHistory) {
	$GLOBALS['ext_ver_history'] = (array) $verHistory;
}

// Getter for EXT_VER_HISTORY array
function EXT_GET_VER_HISTORY () {
	return $GLOBALS['ext_ver_history'];
}

// Setter for EXT_UPDATE_NOTES flag
function EXT_SET_UPDATE_NOTES ($updateNotes) {
	$GLOBALS['ext_update_notes'] = (string) $updateNotes;
}

// Getter for EXT_UPDATE_NOTES flag
function EXT_GET_UPDATE_NOTES () {
	return $GLOBALS['ext_update_notes'];
}

// Init extension notice
function EXT_INIT_NOTES () {
	$GLOBALS['ext_notes'] = '';
}

// Append extension notice
function EXT_APPEND_NOTES ($notes) {
	$GLOBALS['ext_notes'] .= (string) $notes;
}

// Getter for extension notes
function EXT_GET_NOTES () {
	return $GLOBALS['ext_notes'];
}

// Setter for current extension name
function EXT_SET_CURR_NAME ($ext_name) {
	$GLOBALS['curr_ext_name'] = (string) $ext_name;
}

// Getter for current extension name
function EXT_GET_CURR_NAME () {
	if (isset($GLOBALS['curr_ext_name'])) {
		return $GLOBALS['curr_ext_name'];
	} // END - if

	// Not set!
	debug_report_bug(__FUNCTION__.": curr_ext_name not initialized. Please execute INIT_EXT_SQLS() before calling this function.");
}

// Init SQLs array for current extension
function INIT_EXT_SQLS () {
	// Auto-init the array now...
	if (!isset($GLOBALS['ext_sqls'][EXT_GET_CURR_NAME()])) {
		$GLOBALS['ext_sqls'][EXT_GET_CURR_NAME()] = array();
	} // END - if
}

// Adds SQLs to the SQLs array but "assigns" it with current extension name
function ADD_EXT_SQL ($sql) {
	$GLOBALS['ext_sqls'][EXT_GET_CURR_NAME()][] = $sql;
}

// Getter for SQLs array for current extension
function GET_EXT_SQLS () {
	// Output debug backtrace if not found (SHOULD NOT HAPPEN!)
	if (!isset($GLOBALS['ext_sqls'][EXT_GET_CURR_NAME()])) {
		// Not found, should not happen
		debug_report_bug(sprintf("ext_sqls is empty, current extension: %s",
		EXT_GET_CURR_NAME()
		));
	} // END - if

	// Return the array
	return $GLOBALS['ext_sqls'][EXT_GET_CURR_NAME()];
}

// Removes SQLs for current extension
function UNSET_EXT_SQLS () {
	unset($GLOBALS['ext_sqls'][EXT_GET_CURR_NAME()]);
}

// Auto-initializes the removal list
function EXT_INIT_REMOVAL_LIST () {
	// Is the remove list there?
	if (!isset($GLOBALS['ext_update_remove'])) {
		// Then create it
		$GLOBALS['ext_update_remove'] = array();
	} // END - if
}

// Checks wether the current extension is on the removal list
function EXT_IS_ON_REMOVAL_LIST () {
	// Init removal list
	EXT_INIT_REMOVAL_LIST();

	// Is it there?
	return (in_array(EXT_GET_CURR_NAME(), $GLOBALS['ext_update_remove']));
}

// Adds the current extension to the removal list
function EXT_ADD_CURRENT_TO_REMOVAL_LIST () {
	// Simply add it
	$GLOBALS['ext_update_remove'][] = EXT_GET_CURR_NAME();
}

// Getter for removal list
function EXT_GET_REMOVAL_LIST () {
	// Return the removal list
	return $GLOBALS['ext_update_remove'];
}

// Redirects if the provided extension is not installed
function redirectOnUninstalledExtension ($ext_name) {
	// So is the extension there?
	if ((!isExtensionInstalled($ext_name)) || (!EXT_IS_ACTIVE($ext_name))) {
		// Redirect to index
		redirectToUrl('modules.php?module=index&amp;msg=' . getCode('EXTENSION_PROBLEM') . '&amp;ext=' . $ext_name);
	} // END - if
}

// Determines wether the given extension is installed
function isExtensionInstalled ($ext_name) {
	// Default is not installed
	$isInstalled = false;

	// Check cache ext_version
	$isInstalled = isset($GLOBALS['cache_array']['extensions']['ext_version'][$ext_name]);

	// Return status
	return $isInstalled;
}

//
?>
