<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/25/2003 *
 * ===============                              Last change: 11/29/2005 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : language-functions.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Language functions                               *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Sprachfunktionen                                 *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/
// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), '/inc') + 4) . '/security.php';
	require($INC);
}

// "Getter" for language strings
// @TODO Rewrite all language constants to this function.
function getMessage ($messageId) {
	// Default is not found!
	$return = '!' . $messageId . '!';

	// Is the language string found?
	if (isset($GLOBALS['msg'][strtolower($messageId)])) {
		// Language array element found in small_letters
		$return = $GLOBALS['msg'][$messageId];
	} elseif (isset($GLOBALS['msg'][strtoupper($messageId)])) {
		// @DEPRECATED Language array element found in BIG_LETTERS
		$return = $GLOBALS['msg'][$messageId];
	} elseif (defined($messageId)) {
		// @DEPRECATED Deprecated constant found
		$return = constant($messageId);
	} else {
		// Missing language constant
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Missing message string %s detected.", $messageId));
	}

	// Return the string
	return $return;
}

// "Getter" for language
function getLanguage () {
	// Set default return value to default language from config
	$ret = getConfig('DEFAULT_LANG');

	// Init variable
	$lang = '';

	// Is the variable set
	if (REQUEST_ISSET_GET('mx_lang')) {
		// Accept only first 2 chars
		$lang = substr(REQUEST_GET('mx_lang'), 0, 2);
	} elseif (isset($GLOBALS['cache_array']['language'])) {
		// Use cached
		$ret = $GLOBALS['cache_array']['language'];
	} elseif (!empty($lang)) {
		// Check if main language file does exist
		if (isIncludeReadable('inc/language/' . $lang . '.php')) {
			// Okay found, so let's update cookies
			setLanguage($lang);
		} // END - if
	} elseif (isSessionVariableSet('mx_lang')) {
		// Return stored value from cookie
		$ret = getSession('mx_lang');

		// Fixes a warning before the session has the mx_lang constant
		if (empty($ret)) $ret = getConfig('DEFAULT_LANG');
	}

	// Cache entry
	$GLOBALS['cache_array']['language'] = $ret;

	// Return value
	return $ret;
}

// "Setter" for language
function setLanguage ($lang) {
	// Accept only first 2 chars!
	$lang = substr(SQL_ESCAPE(strip_tags($lang)), 0, 2);

	// Set cookie
	setSession('mx_lang', $lang);
}

// Load the current language file or fixes it to 'de'
// If ext_name is empty, load general language support, else load extension's
// language file.
function loadLanguageFile ($ext_name = '') {
	// Try to get language from session
	$mx_lang = getSession('mx_lang');

	// Set default language if it is not (yet) set
	if (is_null($mx_lang)) $mx_lang = getConfig('DEFAULT_LANG');

	// Generate filename
	if (empty($ext_name)) {
		// Generic
		$languageInclude = sprintf("inc/language/%s.php", SQL_ESCAPE($mx_lang));
	} else {
		// Extension's language file
		$languageInclude = sprintf("inc/language/%s_%s.php", $ext_name, getLanguage());
	}

	// Look for file if no extension name is provided
	if ((empty($ext_name)) && (isIncludeReadable($languageInclude) === false)) {
		// Switch to default (DO NOT CHANGE!!!)
		setLanguage('de');
		$languageInclude = 'inc/language/de.php';

		// And set it temporarily
		setConfigEntry('DEFAULT_LANG', 'de');
	} // END - if

	// Is the file there?
	if (isIncludeReadable($languageInclude)) {
		// Load language file
		loadIncludeOnce($languageInclude);
	} elseif ((isDebugModeEnabled()) && ($ext_name != 'sql_patches') && (substr($ext_name, 0, 10) != 'admintheme')) {
		// No language file is not so good...
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("NOTICE: Extension %s has no language file or we cannot read from it. lang=%s",
			$ext_name, getLanguage()
		));
	}

	// Check for installation mode
	if ((isInstalling()) || (!isInstalled()) || (!isAdminRegistered())) {
		// Load matching language file
		loadInclude('inc/language/install_' . getSession('mx_lang') . '.php');
	} // END - if
}

// [EOF]
?>
